#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef struct {
    char name;
    int sink_time;
    int visited;
} Cave;

int find(char name, Cave* caves, int num_caves) {
    for (int i = 0; i < num_caves; i++) {
        if (name == caves[i].name) {
            return i;
        }
    } 

    return -1;
}

void dump(Cave* caves) {
    for (int i = 0; i < 5; i++) {
        printf("%c: %d %c\n", caves[i].name, caves[i].sink_time, caves[i].visited ? 'X' : ' ');
    }
}

int search(Cave* caves, int** matrix, int num_caves, int current, int target, char* path) {
    /* if we are at the target, then we are done */
    if (current == target) {
        return 1;
    }

    /* drop a bread crumb */
    caves[current].visited = 1;

    /* for all of the caves we can get to from here */
    for (int i = 0; i < num_caves; i++) {
        if (matrix[current][i] && !caves[i].visited && caves[i].sink_time > 1) {
            /* drop all of the counts */
            for (int j = 0; j < num_caves; j++) {
                caves[j].sink_time--;
            }

            /* add it to the path */
            char s[2];
            sprintf(s, "%c", caves[i].name);
            strcat(path, s);

            /* check if we can reach the target through here */
            if (search(caves, matrix, num_caves, i, target, path)) {
                return 1;
            } 

            /* remove it from the path */
            path[strlen(path) - 1] = '\0';

            /* raise all of the counts (to undo this move) */
            for (int j = 0; j < num_caves; j++) {
                caves[j].sink_time++;
            }
        }
    }

    return 0;
}

int main() {
    int num_caves, num_edges, i, j;
    scanf("%d\n", &num_caves);

    /* read in the array of caves */
    Cave* caves = malloc(num_caves * sizeof(Cave));
    for (i = 0; i < num_caves; i++) {
        caves[i].visited = 0;
        scanf("%c %d\n", &caves[i].name, &caves[i].sink_time);
    }

    /* create the matrix and clear it out */
    int** matrix = malloc(num_caves * sizeof(int*));
    for (i = 0; i < num_caves; i++) {
        matrix[i] = malloc(num_caves * sizeof(int));
        for (j = 0; j < num_caves; j++) {
            matrix[i][j] = 0;
        }
    }

    /* read in the edges */
    scanf("%d\n", &num_edges);
    for (i = 0; i < num_edges; i++) {
        char to, from;
        scanf("%c%c\n", &to, &from);
        int a = find(to, caves, num_caves);
        int b = find(from, caves, num_caves);
        matrix[a][b] = 1;
        matrix[b][a] = 1;
    }

    /* search for a way from first Cave to last */
    char path[256];
    sprintf(path, "%c", caves[0].name);

    if (search(caves, matrix, num_caves, 0, num_caves - 1, path)) {
        printf("Aladdin can escape: %s\n", path);
    }
    else {
        printf("Aladdin can not escape!\n");
    }


    /* clean up after ourselves */
    free(caves);
    for (i = 0; i < num_caves; i++) {
        free(matrix[i]);
    }
    free(matrix);

    return 0;
}

